#!/usr/bin/env bash

# Install Script for Debian and Ubuntu installations
# This script can be called multiple times
# Copyright (C) 2018 Andreas Fendt - All Rights Reserved
# Permission to copy and modify is granted under the MIT license
# Last revised 28.04.2018

script=/tmp/install-script
cat > $script << 'EOL'
# configure vim
configure-vim() {
    touch ~/.vimrc
    for config in "filetype plugin indent on" "set tabstop=4" "set shiftwidth=4" \
                  "set expandtab" "syntax enable" "set softtabstop=4" \
                  "set number" "set showcmd" "set cursorline"; do
        grep "$config" ~/.vimrc || echo "$config" >> ~/.vimrc
    done
}

# install fzf
install-fzf() {
    [ ! -d ~/.fzf ]&& cd ~ && git clone --depth 1 https://github.com/junegunn/fzf.git ~/.fzf && ~/.fzf/install
}

# install oh-my-zsh
install-ohmyzsh() {
    [ ! -d ~/.oh-my-zsh ] && (
        sh -c "$(wget https://raw.githubusercontent.com/robbyrussell/oh-my-zsh/master/tools/install.sh -O -)"

        # configure oh-my-zsh
        grep 'ZSH_THEME="agnoster"' ~/.zshrc || sed -i -- 's/ZSH_THEME=".*"/ZSH_THEME="agnoster"/g' ~/.zshrc
        grep "command-not-found" ~/.zshrc || sed -i -- '/^plugins=(/a \\  command-not-found\n  common-aliases\n  sudo\n  systemd\n  tmux\n  ubuntu' ~/.zshrc

        # configure tmux on normal user
        if [[ $EUID -ne 0 ]]; then
            grep "ZSH_TMUX_AUTOSTART" ~/.zshrc || sed -i -- '1s/^/ZSH_TMUX_AUTOSTART="true"\nZSH_TMUX_AUTOQUIT="false"\n\n/' ~/.zshrc
        fi
    )
}

$@
EOL
chmod +x $script

# check if root
if [[ $EUID -ne 0 ]]; then
    echo "This script must be run as root" 1>&2
    exit 1
fi

# check if debian
if [ ! "$(grep -Ei 'debian|buntu' /etc/*release)" ]; then
    echo "This script must be run on a debian or ubuntu system" 1>&2
    exit 1
fi

# update system
apt-get update -y
apt-get dist-upgrade -y
apt-get autoclean -y

# install default packages
apt-get install -y fonts-powerline python-pip zsh sudo mc tree pv vim bash-completion wget git htop iotop iftop nmon apt-file aptitude hexedit tmux
pip install git+https://github.com/jeffkaufman/icdiff.git

# get users
menu=(); while IFS=: read -r name pw uid gid gecos home shell; do if (( uid >= 1000 && uid < 10000 )); then menu+=( "$name" "$uid" on ); fi; done < /etc/passwd
users=$(dialog --checklist "Install this on User" 0 0 4 root "0" on "${menu[@]}" 3>&1 1>&2 2>&3)

# for each user install packages
for user in $users; do
    # install zsh shell
    chsh -s /bin/zsh $user

    # run install and configuration scripts
    for target in install-ohmyzsh install-fzf configure-vim; do
        su -c "$script $target" -s /bin/bash $user
    done
done

# configure tmux
for config in "bind '\"' split-window -c \"#{pane_current_path}\"" \
              "bind % split-window -h -c \"#{pane_current_path}\"" \
              "bind c new-window -c \"#{pane_current_path}\""; do
    grep "$config" /etc/tmux.conf || echo "$config" >> /etc/tmux.conf
done

# configure system
apt-get install -y console-data console-setup locales \
                   keyboard-configuration tzdata
dpkg-reconfigure console-data console-setup locales \
                 keyboard-configuration tzdata

# run programs
apt-file update
sensors-detect --auto

# delete temporary script file
rm -f $script
